"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createBridgeInvocationHandler = void 0;
var extract_1 = require("./extract");
var createBridgeInvocationHandler = function (sourceManager) {
    return function (method, args) {
        var _a, _b, _c, _d;
        switch (method) {
            case 'didStartRecording': {
                return sourceManager.onStartRecording((0, extract_1.extractOptions)(args));
            }
            case 'didStopRecording':
                return sourceManager.onStopRecording();
            case 'sessionDidStart': {
                var sessionId = (0, extract_1.extractString)(args, 'sessionId');
                var timestamp = (_a = (0, extract_1.extractTimestamp)(args)) !== null && _a !== void 0 ? _a : new Date();
                var foregrounded = (_b = (0, extract_1.extractBoolean)(args, 'foregrounded')) !== null && _b !== void 0 ? _b : false;
                if (sessionId !== null) {
                    return sourceManager.onSessionStart(sessionId, timestamp, foregrounded);
                }
                else {
                    return Promise.reject('sessionDidStart was called with an invalid session Id.');
                }
            }
            case 'applicationDidEnterForeground': {
                var timestamp = (_c = (0, extract_1.extractTimestamp)(args)) !== null && _c !== void 0 ? _c : new Date();
                return sourceManager.onApplicationForegrounded(timestamp);
            }
            case 'applicationDidEnterBackground': {
                var timestamp = (_d = (0, extract_1.extractTimestamp)(args)) !== null && _d !== void 0 ? _d : new Date();
                return sourceManager.onApplicationBackgrounded(timestamp);
            }
            default:
                return Promise.reject("Unknown method ".concat(method, "."));
        }
    };
};
exports.createBridgeInvocationHandler = createBridgeInvocationHandler;
