"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createCoreMethods = void 0;
var logger_1 = require("../logger");
var sanitize_1 = require("./sanitize");
var createCoreMethods = function (handleInvocation, logger, defaultSource, resolvePageviewKey) {
    if (defaultSource === void 0) { defaultSource = null; }
    var logSource = defaultSource === null || defaultSource === void 0 ? void 0 : defaultSource.name;
    var sanitizedDefaultSource = (0, sanitize_1.sanitizeSourceInfo)(defaultSource, logger, logSource);
    return {
        startRecording: function (environmentId, options) {
            if (arguments.length === 0) {
                logger.warn('startRecording failed because environmentId was omitted.', logSource);
                return;
            }
            if (!(0, sanitize_1.isNonEmptyString)(environmentId)) {
                logger.warn('startRecording failed because environmentId was not a string.', logSource);
                return;
            }
            handleInvocation('startRecording', { environmentId: environmentId, options: options || {} });
        },
        stopRecording: function () {
            handleInvocation('stopRecording', {});
        },
        track: function (event, properties, timestamp, sourceInfo, pageview) {
            return __awaiter(this, void 0, void 0, function () {
                var sanitizedProperties, sanitizedSourceInfo, javascriptEpochTimestamp, pageviewKey, args;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            timestamp !== null && timestamp !== void 0 ? timestamp : (timestamp = new Date());
                            if (!(0, sanitize_1.isNonEmptyString)(event)) {
                                logger.warn('track failed because event was not a string.', logSource);
                                return [2 /*return*/];
                            }
                            sanitizedProperties = (0, sanitize_1.sanitizeProperties)(properties, logger, logSource);
                            sanitizedSourceInfo = (0, sanitize_1.sanitizeSourceInfo)(sourceInfo, logger, logSource) || sanitizedDefaultSource;
                            javascriptEpochTimestamp = (0, sanitize_1.sanitizeTimestamp)(timestamp, logger, logSource);
                            return [4 /*yield*/, resolvePageviewKey(pageview, sourceInfo, timestamp)];
                        case 1:
                            pageviewKey = _a.sent();
                            args = { event: event };
                            if (sanitizedProperties) {
                                args.properties = sanitizedProperties;
                            }
                            if (javascriptEpochTimestamp) {
                                args.javascriptEpochTimestamp = javascriptEpochTimestamp;
                            }
                            if (sanitizedSourceInfo) {
                                args.sourceLibrary = sanitizedSourceInfo;
                            }
                            if (pageviewKey) {
                                args.pageviewKey = pageviewKey;
                            }
                            handleInvocation('track', args);
                            return [2 /*return*/];
                    }
                });
            });
        },
        identify: function (identity) {
            if (arguments.length === 0) {
                logger.warn('identify failed because identity was omitted.', logSource);
                return;
            }
            if (!(0, sanitize_1.isNonEmptyString)(identity)) {
                logger.warn('identify failed because identity was not a string.', logSource);
                return;
            }
            handleInvocation('identify', { identity: identity });
        },
        resetIdentity: function () {
            handleInvocation('resetIdentity', {});
        },
        addUserProperties: function (properties) {
            var sanitizedProperties = (0, sanitize_1.sanitizeProperties)(properties, logger, logSource);
            if (!sanitizedProperties) {
                return; // Already logged.
            }
            if (Object.keys(sanitizedProperties).length === 0) {
                logger.warn('addUserProperties was called without any valid properties.');
                return;
            }
            handleInvocation('addUserProperties', { properties: sanitizedProperties });
        },
        addEventProperties: function (properties) {
            var sanitizedProperties = (0, sanitize_1.sanitizeProperties)(properties, logger, logSource);
            if (!sanitizedProperties) {
                return; // Already logged.
            }
            if (Object.keys(sanitizedProperties).length === 0) {
                logger.warn('addEventProperties was called without any valid properties.');
                return;
            }
            handleInvocation('addEventProperties', { properties: sanitizedProperties });
        },
        removeEventProperty: function (name) {
            if (arguments.length === 0) {
                logger.warn('removeEventProperty failed because name was omitted.', logSource);
                return;
            }
            if (!(0, sanitize_1.isNonEmptyString)(name)) {
                logger.warn('removeEventProperty failed because name was not a string.', logSource);
                return;
            }
            handleInvocation('removeEventProperty', { name: name });
        },
        clearEventProperties: function () {
            handleInvocation('clearEventProperties', {});
        },
        setLogLevel: function (logLevel) {
            logger.setLogLevel(logLevel);
        },
        setLogChannel: function (logChannel) {
            logger.setLogChannel(logChannel);
        },
        logToConsole: function () {
            this.setLogChannel(logger_1.HeapConsoleLogChannel);
        },
        getEnvironmentId: function () {
            return handleInvocation('environmentId', {});
        },
        getSessionId: function () {
            return handleInvocation('sessionId', {});
        },
        fetchSessionId: function () {
            return handleInvocation('fetchSessionId', {});
        },
        getUserId: function () {
            return handleInvocation('userId', {});
        },
        getIdentity: function () {
            return handleInvocation('identity', {});
        },
    };
};
exports.createCoreMethods = createCoreMethods;
