import type { HeapLogger } from '../logger';
import { SourceManager } from './sourcemanager';
import type { Pageview, PageviewParameter, SourceInfo } from './types';
export declare function addDeadKey(pageviewKey: string): void;
export declare function removeDeadKeys(removedKeys: Array<string>): void;
export declare function getDeadKeys(): string[];
/**
 * Sets the last pageview to the results of resolution and adds the pageview to the known key registry.
 *
 * @param promise A promise resolving to a pageview or null.
 */
export declare function setLastPageview(promise: Promise<Pageview | null>): void;
export declare function clearLastPageview(): void;
export declare function getLastPageview(): Promise<Pageview<any> | null>;
/**
 * Gets a recently resolved pageview if it exists.
 * @param pageviewKey
 * @returns The most recent matching pageview, or null.
 */
export declare const getRecentResolutionForKey: (pageviewKey: string) => Pageview | null;
/**
 * Adds a pageview to the recent resolution list and schedules its removal after a few seconds.
 *
 * @param pageview The pageview to add
 */
export declare const recordRecentPageviewResolution: (pageview: Pageview) => void;
/**
 * Sets the timeout resolution.
 *
 * This is used in testing since the default timeout is fairly high.
 *
 * @param timeout The number of milliseconds before pageviews are removed from the list.
 */
export declare const setRecentPageviewResolutionTimeout: (timeout: number) => void;
/**
 * Clears all recently pageview resolutions.
 *
 * This is for use in `afterEach` so tests don't persist state and so timeouts can be resolved before jest exits.
 */
export declare const clearRecentPageviewResolutions: () => void;
/**
 * Waits for all currently resolved pageviews to be removed from the list.
 * @returns A promise that resolves when the pageviews are removed.
 */
export declare const waitForRecentResolutionsToTimeout: () => Promise<void[]>;
/**
 * A function that resolves a pageview for `track` and `trackInteraction` based on parameters from those methods.
 */
export type PageviewKeyResolver = (pageview: PageviewParameter | null | undefined, source: SourceInfo | null | undefined, timestamp: Date) => Promise<string | null>;
/**
 * Creates a function that can resolve a pageview for a given interaction.
 */
export declare const createPageviewKeyResolver: (sourceManager: SourceManager | null | undefined, logger: HeapLogger, logSource: string | undefined) => PageviewKeyResolver;
