import { HeapLogger } from '../logger';
import { InvocationHandler } from '../bridge';
import { Options, Pageview, Source, SourceManagementApi } from './types';
/**
 * `SourceManager` consumes events from a bridge invocation handler and routes them to registered sources.
 *
 * It also keeps track of state as inferred from the method calls to replay messages on `addSource` and
 * support pageview resolution.
 */
export declare class SourceManager {
    sources: Record<string, Source>;
    defaultSource: Source | null;
    sessionId: string | null;
    options: Options | null;
    foregrounded: boolean;
    logger: HeapLogger;
    logSource: string | undefined;
    constructor(logger: HeapLogger, logSource?: string | undefined);
    addSource(source: Source, isDefault: boolean, timestamp: Date): void;
    removeSource(name: string): void;
    removeAll(): void;
    onStartRecording(options: Options): Promise<void>;
    private callOnStartRecording;
    onStopRecording(): Promise<void>;
    private callOnStopRecording;
    onSessionStart(sessionId: string, timestamp: Date, foregrounded: boolean): Promise<void>;
    private callOnSessionStart;
    onApplicationForegrounded(timestamp: Date): Promise<void>;
    private callOnApplicationForegrounded;
    onApplicationBackgrounded(timestamp: Date): Promise<void>;
    private callOnApplicationBackgrounded;
    activePageview(sourceName: string | null | undefined, timestamp: Date): Promise<Pageview<any> | null>;
    private callActivePageview;
    pageviewNeedsReissue(pageview: Pageview | null | undefined): pageview is Pageview;
    reissuePageview(pageview: Pageview, sessionId: string | null | undefined, timestamp: Date): Promise<Pageview<any> | null>;
    reissuePageviewWithKey(pageviewKey: string, sessionId: string, timestamp: Date): Promise<Pageview<any> | null>;
    private callReissuePageview;
    private swallowErrors;
}
export declare const createSourceManagementMethods: (handleInvocation: InvocationHandler, sourceManager: SourceManager | null) => SourceManagementApi;
