"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createSourceManagementMethods = exports.SourceManager = void 0;
var pageviews_1 = require("./pageviews");
/**
 * `SourceManager` consumes events from a bridge invocation handler and routes them to registered sources.
 *
 * It also keeps track of state as inferred from the method calls to replay messages on `addSource` and
 * support pageview resolution.
 */
var SourceManager = /** @class */ (function () {
    function SourceManager(logger, logSource) {
        if (logSource === void 0) { logSource = undefined; }
        this.sources = {};
        this.defaultSource = null;
        this.sessionId = null;
        this.options = null;
        this.foregrounded = false;
        this.logger = logger;
        this.logSource = logSource;
    }
    SourceManager.prototype.addSource = function (source, isDefault, timestamp) {
        var _a;
        var oldSource = this.sources[source.name];
        var removedDefault;
        if (oldSource === source) {
            return; // The source was already registered.
        }
        this.sources[source.name] = source;
        if (isDefault) {
            removedDefault = this.defaultSource;
            this.defaultSource = source;
        }
        else if (((_a = this.defaultSource) === null || _a === void 0 ? void 0 : _a.name) === source.name) {
            removedDefault = this.defaultSource;
            this.defaultSource = null;
        }
        else {
            removedDefault = null;
        }
        if (removedDefault) {
            if (isDefault) {
                this.logger.warn("Replaced default source ".concat(removedDefault.name, " with ").concat(source.name, "."), this.logSource);
            }
            else {
                this.logger.warn("Removed default source ".concat(removedDefault.name, " when setting a new non-default source of the same name."), this.logSource);
            }
        }
        if (this.options) {
            if (oldSource) {
                this.callOnStopRecording(oldSource);
            }
            this.callOnStartRecording(source, this.options);
            if (this.sessionId) {
                this.callOnSessionStart(source, this.sessionId, timestamp, this.foregrounded);
            }
            if (this.foregrounded) {
                this.callOnApplicationForegrounded(source, timestamp);
            }
        }
    };
    SourceManager.prototype.removeSource = function (name) {
        var _a;
        var oldSource = this.sources[name];
        delete this.sources[name];
        if (((_a = this.defaultSource) === null || _a === void 0 ? void 0 : _a.name) === name) {
            this.defaultSource = null;
        }
        if (this.options && oldSource) {
            this.callOnStopRecording(oldSource);
        }
    };
    SourceManager.prototype.removeAll = function () {
        var e_1, _a;
        try {
            for (var _b = __values(Object.keys(this.sources)), _c = _b.next(); !_c.done; _c = _b.next()) {
                var sourceName = _c.value;
                this.removeSource(sourceName);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    SourceManager.prototype.onStartRecording = function (options) {
        return __awaiter(this, void 0, void 0, function () {
            var _a, _b, source, e_2_1;
            var e_2, _c;
            return __generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        this.options = options;
                        this.sessionId = null;
                        _d.label = 1;
                    case 1:
                        _d.trys.push([1, 6, 7, 8]);
                        _a = __values(Object.values(this.sources)), _b = _a.next();
                        _d.label = 2;
                    case 2:
                        if (!!_b.done) return [3 /*break*/, 5];
                        source = _b.value;
                        return [4 /*yield*/, this.callOnStartRecording(source, options)];
                    case 3:
                        _d.sent();
                        _d.label = 4;
                    case 4:
                        _b = _a.next();
                        return [3 /*break*/, 2];
                    case 5: return [3 /*break*/, 8];
                    case 6:
                        e_2_1 = _d.sent();
                        e_2 = { error: e_2_1 };
                        return [3 /*break*/, 8];
                    case 7:
                        try {
                            if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
                        }
                        finally { if (e_2) throw e_2.error; }
                        return [7 /*endfinally*/];
                    case 8: return [2 /*return*/];
                }
            });
        });
    };
    SourceManager.prototype.callOnStartRecording = function (source, options) {
        return this.swallowErrors(function () { var _a; return (_a = source.onStartRecording) === null || _a === void 0 ? void 0 : _a.call(source, options); }, source.name, 'onStartRecording', function () { return Promise.resolve(); });
    };
    SourceManager.prototype.onStopRecording = function () {
        return __awaiter(this, void 0, void 0, function () {
            var _a, _b, source, e_3_1;
            var e_3, _c;
            return __generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        this.options = null;
                        this.sessionId = null;
                        _d.label = 1;
                    case 1:
                        _d.trys.push([1, 6, 7, 8]);
                        _a = __values(Object.values(this.sources)), _b = _a.next();
                        _d.label = 2;
                    case 2:
                        if (!!_b.done) return [3 /*break*/, 5];
                        source = _b.value;
                        return [4 /*yield*/, this.callOnStopRecording(source)];
                    case 3:
                        _d.sent();
                        _d.label = 4;
                    case 4:
                        _b = _a.next();
                        return [3 /*break*/, 2];
                    case 5: return [3 /*break*/, 8];
                    case 6:
                        e_3_1 = _d.sent();
                        e_3 = { error: e_3_1 };
                        return [3 /*break*/, 8];
                    case 7:
                        try {
                            if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
                        }
                        finally { if (e_3) throw e_3.error; }
                        return [7 /*endfinally*/];
                    case 8: return [2 /*return*/];
                }
            });
        });
    };
    SourceManager.prototype.callOnStopRecording = function (source) {
        return this.swallowErrors(function () { var _a; return (_a = source.onStopRecording) === null || _a === void 0 ? void 0 : _a.call(source); }, source.name, 'onStopRecording', function () { return Promise.resolve(); });
    };
    SourceManager.prototype.onSessionStart = function (sessionId, timestamp, foregrounded) {
        return __awaiter(this, void 0, void 0, function () {
            var _a, _b, source, e_4_1;
            var e_4, _c;
            return __generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        this.sessionId = sessionId;
                        this.foregrounded = foregrounded;
                        _d.label = 1;
                    case 1:
                        _d.trys.push([1, 6, 7, 8]);
                        _a = __values(Object.values(this.sources)), _b = _a.next();
                        _d.label = 2;
                    case 2:
                        if (!!_b.done) return [3 /*break*/, 5];
                        source = _b.value;
                        return [4 /*yield*/, this.callOnSessionStart(source, sessionId, timestamp, foregrounded)];
                    case 3:
                        _d.sent();
                        _d.label = 4;
                    case 4:
                        _b = _a.next();
                        return [3 /*break*/, 2];
                    case 5: return [3 /*break*/, 8];
                    case 6:
                        e_4_1 = _d.sent();
                        e_4 = { error: e_4_1 };
                        return [3 /*break*/, 8];
                    case 7:
                        try {
                            if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
                        }
                        finally { if (e_4) throw e_4.error; }
                        return [7 /*endfinally*/];
                    case 8: return [2 /*return*/];
                }
            });
        });
    };
    SourceManager.prototype.callOnSessionStart = function (source, sessionId, timestamp, foregrounded) {
        return this.swallowErrors(function () { var _a; return (_a = source.onSessionStart) === null || _a === void 0 ? void 0 : _a.call(source, sessionId, timestamp, foregrounded); }, source.name, 'onSessionStart', function () { return Promise.resolve(); });
    };
    SourceManager.prototype.onApplicationForegrounded = function (timestamp) {
        return __awaiter(this, void 0, void 0, function () {
            var _a, _b, source, e_5_1;
            var e_5, _c;
            return __generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        this.foregrounded = true;
                        _d.label = 1;
                    case 1:
                        _d.trys.push([1, 6, 7, 8]);
                        _a = __values(Object.values(this.sources)), _b = _a.next();
                        _d.label = 2;
                    case 2:
                        if (!!_b.done) return [3 /*break*/, 5];
                        source = _b.value;
                        return [4 /*yield*/, this.callOnApplicationForegrounded(source, timestamp)];
                    case 3:
                        _d.sent();
                        _d.label = 4;
                    case 4:
                        _b = _a.next();
                        return [3 /*break*/, 2];
                    case 5: return [3 /*break*/, 8];
                    case 6:
                        e_5_1 = _d.sent();
                        e_5 = { error: e_5_1 };
                        return [3 /*break*/, 8];
                    case 7:
                        try {
                            if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
                        }
                        finally { if (e_5) throw e_5.error; }
                        return [7 /*endfinally*/];
                    case 8: return [2 /*return*/];
                }
            });
        });
    };
    SourceManager.prototype.callOnApplicationForegrounded = function (source, timestamp) {
        return this.swallowErrors(function () { var _a; return (_a = source.onApplicationForegrounded) === null || _a === void 0 ? void 0 : _a.call(source, timestamp); }, source.name, 'onApplicationForegrounded', function () { return Promise.resolve(); });
    };
    SourceManager.prototype.onApplicationBackgrounded = function (timestamp) {
        return __awaiter(this, void 0, void 0, function () {
            var _a, _b, source, e_6_1;
            var e_6, _c;
            return __generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        this.foregrounded = false;
                        _d.label = 1;
                    case 1:
                        _d.trys.push([1, 6, 7, 8]);
                        _a = __values(Object.values(this.sources)), _b = _a.next();
                        _d.label = 2;
                    case 2:
                        if (!!_b.done) return [3 /*break*/, 5];
                        source = _b.value;
                        return [4 /*yield*/, this.callOnApplicationBackgrounded(source, timestamp)];
                    case 3:
                        _d.sent();
                        _d.label = 4;
                    case 4:
                        _b = _a.next();
                        return [3 /*break*/, 2];
                    case 5: return [3 /*break*/, 8];
                    case 6:
                        e_6_1 = _d.sent();
                        e_6 = { error: e_6_1 };
                        return [3 /*break*/, 8];
                    case 7:
                        try {
                            if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
                        }
                        finally { if (e_6) throw e_6.error; }
                        return [7 /*endfinally*/];
                    case 8: return [2 /*return*/];
                }
            });
        });
    };
    SourceManager.prototype.callOnApplicationBackgrounded = function (source, timestamp) {
        return this.swallowErrors(function () { var _a; return (_a = source.onApplicationBackgrounded) === null || _a === void 0 ? void 0 : _a.call(source, timestamp); }, source.name, 'onApplicationBackgrounded', function () { return Promise.resolve(); });
    };
    SourceManager.prototype.activePageview = function (sourceName, timestamp) {
        return __awaiter(this, void 0, void 0, function () {
            var pageview, pageview;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(sourceName && this.sources[sourceName])) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.callActivePageview(this.sources[sourceName], timestamp)];
                    case 1:
                        pageview = _a.sent();
                        if (pageview) {
                            return [2 /*return*/, pageview];
                        }
                        _a.label = 2;
                    case 2:
                        if (!this.defaultSource) return [3 /*break*/, 4];
                        return [4 /*yield*/, this.callActivePageview(this.defaultSource, timestamp)];
                    case 3:
                        pageview = _a.sent();
                        if (pageview) {
                            return [2 /*return*/, pageview];
                        }
                        _a.label = 4;
                    case 4: return [2 /*return*/, null];
                }
            });
        });
    };
    SourceManager.prototype.callActivePageview = function (source, timestamp) {
        var _this = this;
        return this.swallowErrors(function () { var _a; return (_a = source.activePageview) === null || _a === void 0 ? void 0 : _a.call(source, _this.sessionId, timestamp); }, source.name, 'activePageview', function () { return Promise.resolve(null); });
    };
    SourceManager.prototype.pageviewNeedsReissue = function (pageview) {
        return !!(pageview && this.sessionId && this.sessionId !== pageview.sessionId);
    };
    SourceManager.prototype.reissuePageview = function (pageview, sessionId, timestamp) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function () {
            var sourceName, source;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        sourceName = (_a = pageview.sourceInfo) === null || _a === void 0 ? void 0 : _a.name;
                        sessionId !== null && sessionId !== void 0 ? sessionId : (sessionId = (_b = this.sessionId) !== null && _b !== void 0 ? _b : null);
                        if (!sourceName) {
                            return [2 /*return*/, null];
                        }
                        source = this.sources[sourceName];
                        if (!source) {
                            return [2 /*return*/, null];
                        }
                        return [4 /*yield*/, this.callReissuePageview(source, pageview, sessionId, timestamp)];
                    case 1: return [2 /*return*/, _c.sent()];
                }
            });
        });
    };
    SourceManager.prototype.reissuePageviewWithKey = function (pageviewKey, sessionId, timestamp) {
        return __awaiter(this, void 0, void 0, function () {
            var pageview;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        pageview = (0, pageviews_1.getRecentResolutionForKey)(pageviewKey);
                        if (!pageview) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.reissuePageview(pageview, sessionId, timestamp)];
                    case 1: return [2 /*return*/, _a.sent()];
                    case 2: return [2 /*return*/, null];
                }
            });
        });
    };
    SourceManager.prototype.callReissuePageview = function (source, pageview, sessionId, timestamp) {
        return this.swallowErrors(function () { var _a; return (_a = source.reissuePageview) === null || _a === void 0 ? void 0 : _a.call(source, pageview, sessionId, timestamp); }, source.name, 'reissuePageview', function () { return Promise.resolve(null); });
    };
    SourceManager.prototype.swallowErrors = function (fn, sourceName, fnName, fallback) {
        return __awaiter(this, void 0, void 0, function () {
            var value, error_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, fn()];
                    case 1:
                        value = _a.sent();
                        this.logger.trace("Source [".concat(sourceName, "] has completed all work related to ").concat(fnName, "."), this.logSource);
                        return [2 /*return*/, value || fallback()];
                    case 2:
                        error_1 = _a.sent();
                        this.logger.error("Source [".concat(sourceName, "] threw an error while handling ").concat(fnName, ": ").concat(error_1), this.logSource);
                        return [2 /*return*/, fallback()];
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    return SourceManager;
}());
exports.SourceManager = SourceManager;
var createSourceManagementMethods = function (handleInvocation, sourceManager) {
    return {
        addSource: function (source, isDefault, timestamp) {
            handleInvocation('attachRuntimeBridge', {});
            sourceManager === null || sourceManager === void 0 ? void 0 : sourceManager.addSource(source, isDefault !== null && isDefault !== void 0 ? isDefault : false, timestamp !== null && timestamp !== void 0 ? timestamp : new Date());
        },
        removeSource: function (name) {
            sourceManager === null || sourceManager === void 0 ? void 0 : sourceManager.removeSource(name);
        },
    };
};
exports.createSourceManagementMethods = createSourceManagementMethods;
