"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ThrowingSource = exports.CountingSource = void 0;
var CountingSource = /** @class */ (function () {
    function CountingSource(name, version) {
        if (name === void 0) { name = 'CountingSource'; }
        if (version === void 0) { version = '0.0.0'; }
        this.calls = [];
        this.returnsPageviews = true;
        this.name = name;
        this.version = version;
    }
    CountingSource.prototype.clear = function () {
        this.calls.length = 0;
    };
    CountingSource.prototype.reset = function () {
        this.clear();
        this.returnsPageviews = true;
    };
    CountingSource.prototype.onStartRecording = function (options) {
        this.calls.push(['onStartRecording', options]);
        return Promise.resolve();
    };
    CountingSource.prototype.onStopRecording = function () {
        this.calls.push('onStopRecording');
        return Promise.resolve();
    };
    CountingSource.prototype.onSessionStart = function (sessionId, timestamp, foregrounded) {
        this.calls.push(['onSessionStart', sessionId, timestamp, foregrounded]);
        return Promise.resolve();
    };
    CountingSource.prototype.onApplicationForegrounded = function (timestamp) {
        this.calls.push(['onApplicationForegrounded', timestamp]);
        return Promise.resolve();
    };
    CountingSource.prototype.onApplicationBackgrounded = function (timestamp) {
        this.calls.push(['onApplicationBackgrounded', timestamp]);
        return Promise.resolve();
    };
    CountingSource.prototype.reissuePageview = function (pageview, sessionId, timestamp) {
        this.calls.push(['reissuePageview', pageview, sessionId, timestamp]);
        return this.returnsPageviews
            ? Promise.resolve({
                pageviewKey: 'reissue',
                sessionId: sessionId !== null && sessionId !== void 0 ? sessionId : 'default',
                properties: {},
                sourceInfo: { name: this.name, version: this.version, platform: '' },
            })
            : Promise.resolve(null);
    };
    CountingSource.prototype.activePageview = function (sessionId, timestamp) {
        this.calls.push(['activePageview', sessionId, timestamp]);
        return this.returnsPageviews
            ? Promise.resolve({
                pageviewKey: 'active',
                sessionId: sessionId !== null && sessionId !== void 0 ? sessionId : 'default',
                properties: {},
                sourceInfo: { name: this.name, version: this.version, platform: '' },
            })
            : Promise.resolve(null);
    };
    return CountingSource;
}());
exports.CountingSource = CountingSource;
var ThrowingSource = /** @class */ (function () {
    function ThrowingSource(name, version) {
        if (name === void 0) { name = 'ThrowingSource'; }
        if (version === void 0) { version = '0.0.0'; }
        this.name = name;
        this.version = version;
    }
    ThrowingSource.prototype.onStartRecording = function (options) {
        throw 'oops';
    };
    ThrowingSource.prototype.onStopRecording = function () {
        throw 'oops';
    };
    ThrowingSource.prototype.onSessionStart = function (sessionId, timestamp, foregrounded) {
        throw 'oops';
    };
    ThrowingSource.prototype.onApplicationForegrounded = function (timestamp) {
        throw 'oops';
    };
    ThrowingSource.prototype.onApplicationBackgrounded = function (timestamp) {
        throw 'oops';
    };
    ThrowingSource.prototype.reissuePageview = function (pageview, sessionId, timestamp) {
        throw 'oops';
    };
    ThrowingSource.prototype.activePageview = function (sessionId, timestamp) {
        throw 'oops';
    };
    return ThrowingSource;
}());
exports.ThrowingSource = ThrowingSource;
