import { LogChannel, LogLevel } from '../logger';
export type Options = Record<string, any> | {
    baseUrl?: string;
    uploadInterval?: number;
    startSessionImmediately?: boolean;
    clearEventPropertiesOnNewUser?: boolean;
    captureAdvertiserId?: boolean;
    captureVendorId?: boolean;
    disableInteractionTextCapture?: boolean;
    disableInteractionAccessibilityLabelCapture?: boolean;
};
export type PropertyValue = number | string | boolean | bigint;
export type Properties = Record<string, PropertyValue>;
export interface SourceInfo {
    name: string;
    version: string;
    platform: string;
    properties?: Properties;
}
export declare function isPropertyValue(value: any): value is PropertyValue;
export interface PageviewProperties {
    componentOrClassName?: string;
    title?: string;
    url?: string;
    sourceProperties?: Properties;
}
export interface Pageview<T = any> {
    pageviewKey: string;
    sessionId: string;
    properties: PageviewProperties;
    userInfo?: T;
    sourceInfo?: SourceInfo | null;
}
export type PageviewParameter = Promise<Pageview | null> | 'none' | Pageview | null;
export type InteractionType = 'unspecified' | 'click' | 'touch' | 'change' | 'submit' | {
    builtin: number;
} | {
    custom: string;
};
export interface InteractionNode {
    nodeName: string;
    nodeText?: string;
    nodeHtmlClass?: string;
    nodeId?: string;
    href?: string;
    accessibilityLabel?: string;
    attributes?: Record<string, string>;
    referencingPropertyName?: string;
}
export interface Source {
    name: string;
    version: string;
    onStartRecording?: (options: Options) => Promise<void>;
    onStopRecording?: () => Promise<void>;
    onSessionStart?: (sessionId: string, timestamp: Date, foregrounded: boolean) => Promise<void>;
    onApplicationForegrounded?: (timestamp: Date) => Promise<void>;
    onApplicationBackgrounded?: (timestamp: Date) => Promise<void>;
    activePageview?: (sessionId: string | null, timestamp: Date) => Promise<Pageview | null>;
    reissuePageview?: (pageview: Pageview, sessionId: string | null, timestamp: Date) => Promise<Pageview | null>;
}
export interface SourceManagementApi {
    addSource(source: Source, isDefault?: boolean, timestamp?: Date): void;
    removeSource(name: string): void;
}
export interface AutocaptureApi {
    trackPageview<T = any>(properties: PageviewProperties, timestamp?: Date | null, sourceInfo?: SourceInfo | null, userInfo?: T): Promise<Pageview | null>;
    trackInteraction(interaction: InteractionType, nodes: Array<InteractionNode>, callbackName?: string | null, timestamp?: Date | null, sourceInfo?: SourceInfo | null, sourceProperties?: Properties | null, pageview?: Pageview | 'none' | null): Promise<void>;
}
export interface CoreApi {
    startRecording(environmentId: string, options?: Options): void;
    stopRecording(): void;
    track(event: string, properties?: Properties | null, timestamp?: Date | null, sourceInfo?: SourceInfo | null, pageview?: Pageview | 'none' | null): Promise<void>;
    identify(identity: string): void;
    resetIdentity(): void;
    addUserProperties(properties: Properties): void;
    addEventProperties(properties: Properties): void;
    removeEventProperty(name: string): void;
    clearEventProperties(): void;
    setLogLevel(logLevel: LogLevel): void;
    setLogChannel(logChannel: LogChannel): void;
    logToConsole(): void;
    getEnvironmentId(): Promise<string | null>;
    getSessionId(): Promise<string | null>;
    fetchSessionId(): Promise<string | null>;
    getUserId(): Promise<string | null>;
    getIdentity(): Promise<string | null>;
}
export type Heap = CoreApi & AutocaptureApi & SourceManagementApi;
