"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HeapConsoleLogChannel = exports.createLogger = exports.LogLevel = void 0;
var sanitize_1 = require("../heap/sanitize");
var LogLevel;
(function (LogLevel) {
    LogLevel["None"] = "none";
    LogLevel["Error"] = "error";
    LogLevel["Warn"] = "warn";
    LogLevel["Info"] = "info";
    LogLevel["Debug"] = "debug";
    LogLevel["Trace"] = "trace";
})(LogLevel = exports.LogLevel || (exports.LogLevel = {}));
function log(handleInvocation, message, logLevel, source) {
    if (!(0, sanitize_1.isNonEmptyString)(message)) {
        return;
    }
    var args = {
        logLevel: logLevel,
        message: message,
    };
    if ((0, sanitize_1.isNonEmptyString)(source)) {
        args.source = source;
    }
    handleInvocation('heapLogger_log', args);
}
var createLogger = function (handleInvocation) {
    var channel = null;
    return {
        error: function (message, source) {
            log(handleInvocation, message, LogLevel.Error, source);
        },
        warn: function (message, source) {
            log(handleInvocation, message, LogLevel.Warn, source);
        },
        info: function (message, source) {
            log(handleInvocation, message, LogLevel.Info, source);
        },
        debug: function (message, source) {
            log(handleInvocation, message, LogLevel.Debug, source);
        },
        trace: function (message, source) {
            log(handleInvocation, message, LogLevel.Trace, source);
        },
        setLogLevel: function (logLevel) {
            handleInvocation('heapLogger_setLogLevel', { logLevel: logLevel });
        },
        getLogLevel: function () {
            return handleInvocation('heapLogger_logLevel', {});
        },
        getLogChannel: function () {
            return channel;
        },
        setLogChannel: function (logChannel) {
            channel = logChannel;
            if (logChannel) {
                handleInvocation('attachLogChannel', {});
            }
        },
    };
};
exports.createLogger = createLogger;
var HeapConsoleLogChannel = function (logLevel, message, source) {
    if (!message) {
        return;
    }
    var text = source ? "[".concat(source, "] ").concat(message) : message;
    switch (logLevel) {
        case LogLevel.Error:
            console.error(text);
            break;
        case LogLevel.Info:
            // eslint-disable-next-line no-console
            console.info(text);
            break;
        case LogLevel.Warn:
            // eslint-disable-next-line no-console
            console.warn(text);
            break;
        case LogLevel.Debug:
            // eslint-disable-next-line no-console
            console.debug(text);
            break;
        case LogLevel.Trace:
            // eslint-disable-next-line no-console
            console.trace(text);
            break;
        default:
            break;
    }
};
exports.HeapConsoleLogChannel = HeapConsoleLogChannel;
